@@ -10,12 +10,21 @@ Perlbal - Reverse-proxy load balancer and webserver
 
 =head1 SEE ALSO
 
- http://www.danga.com/perlbal/
+L<http://www.danga.com/perlbal/>
+
+=head1 CONTRIBUTING
+
+Got a patch?  Or a bug report?  Instructions on how to contribute
+are located here:
+
+L<http://contributing.appspot.com/perlbal>
+
+Thanks!
 
 =head1 COPYRIGHT AND LICENSE
 
 Copyright 2004, Danga Interactive, Inc.
-Copyright 2005-2007, Six Apart, Ltd.
+Copyright 2005-2010, Six Apart, Ltd.
 
 You can use and redistribute Perlbal under the same terms as Perl itself.
 
@@ -30,10 +39,10 @@ BEGIN {
 
 my $has_gladiator  = eval "use Devel::Gladiator; 1;";
 my $has_cycle      = eval "use Devel::Cycle; 1;";
-use Devel::Peek;
+my $has_devel_peek = eval "use Devel::Peek; 1;";
 
 use vars qw($VERSION);
-$VERSION = '1.73';
+$VERSION = '1.80';
 
 use constant DEBUG => $ENV{PERLBAL_DEBUG} || 0;
 use constant DEBUG_OBJ => $ENV{PERLBAL_DEBUG_OBJ} || 0;
@@ -43,7 +52,6 @@ use strict;
 use warnings;
 no  warnings qw(deprecated);
 
-use Storable ();
 use IO::Socket;
 use IO::Handle;
 use IO::File;
@@ -60,7 +68,7 @@ $Perlbal::syslog_open = 0;
 use Getopt::Long;
 use Carp qw(cluck croak);
 use Errno qw(EBADF);
-use POSIX ();
+use POSIX qw(SIG_BLOCK SIG_UNBLOCK SIGINT sigprocmask);
 
 our(%TrackVar);
 sub track_var {
@@ -117,6 +125,13 @@ if ($ENV{PERLBAL_XS_HEADERS} && $XSModules{headers}) {
     Perlbal::XS::HTTPHeaders::enable();
 }
 
+# unactivate field::new 
+if ($ENV{PERLBAL_REMOVE_FIELDS}) {
+	use Perlbal::Fields;
+	Perlbal::Fields->remove();
+}
+
+
 # setup a USR1 signal handler that tells us to dump some basic statistics
 # of how we're doing to the syslog
 $SIG{'USR1'} = sub {
@@ -198,7 +213,7 @@ sub service {
 sub create_service {
     my $class = shift;
     my $name = shift;
-    
+
     unless (defined($name)) {
         $name = "____auto_".($service_autonumber++);
     }
@@ -257,14 +272,15 @@ sub run_manage_command {
     $cmd =~ s/\s+$//;
     $cmd =~ s/\s+/ /g;
 
-    my $orig = $cmd; # save original case for some commands
-    $cmd =~ s/^([^=]+)/lc $1/e; # lowercase everything up to an =
-    return 1 unless $cmd =~ /^\S/;
-
     # expand variables
+    my $orig = $cmd; # save original case for some commands
+
     $cmd =~ s/\$\{(.+?)\}/_expand_config_var($1)/eg;
     $cmd =~ s/\$(\w+)/$ENV{$1}/g;
 
+    $cmd =~ s/^([^=]+)/lc $1/e; # lowercase everything up to an =
+    return 1 unless $cmd =~ /^\S/;
+
     $out ||= sub {};
     $ctx ||= Perlbal::CommandContext->new;
 
@@ -329,7 +345,7 @@ sub arena_ref_counts {
 my %last_gladiator;
 sub MANAGE_gladiator {
     my $mc = shift->no_opts;
-    unless ($has_gladiator) {
+    unless ($has_gladiator && $has_devel_peek) {
         $mc->end;
         return;
     }
@@ -470,7 +486,7 @@ sub MANAGE_mime {
             return $mc->err("$arg1 not a defined extension.");
         }
     } else {
-        return $mc->err("Usage: list, remove <ext>, add <ext> <mime>");
+        return $mc->err("Usage: list, remove <ext>, set <ext> <mime>");
     }
 }
 
@@ -1070,6 +1086,14 @@ sub MANAGE_pool {
     return $mc->ok;
 }
 
+sub MANAGE_default {
+    my $mc = shift->parse(qr/^default (\w+) ?= ?(.+)$/,
+                          "usage: DEFAULT <param> = <value>");
+
+    my ($key, $val) = $mc->args;
+    return Perlbal::Service::set_defaults($mc, $key => $val);
+}
+
 sub MANAGE_set {
     my $mc = shift->parse(qr/^set (?:(\w+)[\. ])?([\w\.]+) ?= ?(.+)$/,
                           "usage: SET [<service>] <param> = <value>");
@@ -1126,22 +1150,46 @@ sub MANAGE_load {
     my $last_case;
     my $last_class;
 
-    my $load = sub {
-        my $name = shift;
+    my $good_error;
+
+    # TODO case protection
+
+    foreach my $name ($fn, lc $fn, ucfirst lc $fn) {
         $last_case = $name;
         my $class = $last_class = "Perlbal::Plugin::$name";
-        my $rv = eval "use $class; $class->load; 1;";
-        return $mc->err($@) if ! $rv && $@ !~ /^Can\'t locate/;
-        return $rv;
-    };
+        my $file = $class . ".pm";
+        $file =~ s!::!/!g;
 
-    my $rv = $load->($fn) || $load->(lc $fn) || $load->(ucfirst lc $fn);
-    return $mc->err($@) unless $rv;
+        my $rv = eval "use $class; $class->can('load');";
 
-    $PluginCase{lc $fn} = $last_case;
-    $plugins{$last_case} = $last_class;
+        if ($rv) {
+            $good_error = undef;
+            last;
+        }
 
-    return $mc->ok;
+        # If we don't have a good error yet, start with this one.
+        $good_error = $@ unless defined $good_error;
+
+        # If the file existed perl will place an entry in %INC (though it will be undef due to compilation error)
+        if ($@ and exists $INC{$file}) {
+            $good_error = $@;
+            last;
+        }
+    }
+
+    unless (defined $good_error) {
+        my $rv = eval "$last_class->load; 1;";
+
+        if ($rv) {
+            $PluginCase{lc $fn} = $last_case;
+            $plugins{$last_case} = $last_class;
+            return $mc->ok;
+        }
+
+        $good_error = $@;
+    }
+
+    return $mc->err($good_error);
 }
 
 sub MANAGE_reload {
@@ -1229,16 +1277,22 @@ sub daemonize {
     IO::AIO::max_parallel(0)
         if $Perlbal::OPTMOD_IO_AIO;
 
+    my $sigset = POSIX::SigSet->new(SIGINT);
+    sigprocmask(SIG_BLOCK, $sigset)
+        or die "Can't block sigint for fork: $!";
+
     ## Fork and exit parent
     if ($pid = fork) { exit 0; }
 
+    sigprocmask(SIG_UNBLOCK, $sigset)
+        or die "Can't unblock sigint after fork: $!";
+
     ## Detach ourselves from the terminal
     croak "Cannot detach from controlling terminal"
         unless $sess_id = POSIX::setsid();
 
-    ## Prevent possibility of acquiring a controlling terminal
-    $SIG{'HUP'} = 'IGNORE';
-    if ($pid = fork) { exit 0; }
+    # Handler for INT needs to be restored.
+    $SIG{INT} = 'DEFAULT';
 
     ## Change working directory
     chdir "/";
@@ -1335,7 +1389,12 @@ sub log {
     if ($foreground) {
         # syslog acts like printf so we have to use printf and append a \n
         shift; # ignore the first parameter (info, warn, crit, etc)
-        printf(shift(@_) . "\n", @_);
+        my $message = shift;
+        if (@_) {
+            printf("$message\n", @_);
+        } else {
+            print("$message\n");
+        }
     } else {
         # just pass the parameters to syslog
         Sys::Syslog::syslog(@_) if $Perlbal::syslog_open;
